use crate::vm::object::DvmObject;
use crate::vm::value::DvmValue;
use crate::vm::{DvmContext, DvmState};
use std::cell::RefCell;
use std::rc::Rc;

/// ```deimos
/// // Signature
/// print(m: &(Byte | Int | Long | Double | USize | Boolean | Object | Empty))
/// ```
pub fn dm_print(state: &mut DvmState, context: &DvmContext) {
    print!("{}", get_string(state.get_stack_argument(0)));
}

pub fn dm_println(state: &mut DvmState, context: &DvmContext) {
    println!("{}", get_string(state.get_stack_argument(0)));
}

fn get_string(dvm_value: DvmValue) -> String {
    match dvm_value {
        DvmValue::Byte(b) => b.to_string(),
        DvmValue::Int(i) => i.to_string(),
        DvmValue::Long(l) => l.to_string(),
        DvmValue::Double(d) => d.to_string(),
        DvmValue::Boolean(b) => b.to_string(),
        DvmValue::Object(object) => object_to_string(object.clone()),
        DvmValue::USize(u) => u.to_string(),
        DvmValue::Array(a) => a.borrow().to_string(),
        DvmValue::String(s) => s.to_string(),
        DvmValue::Empty => String::from("Empty"),
    }
}

fn object_to_string(object: Rc<RefCell<DvmObject>>) -> String {
    todo!("Lookup to_string method!")
}
